@@ -3,7 +3,7 @@ use pyo3::exceptions::PyModuleNotFoundError;
 use pyo3::prelude::*;
 
 /// A prober that can detect remote Git repositories.
-pub struct RemoteGitProber(PyObject);
+pub struct RemoteGitProber(Py<PyAny>);
 
 /// The SHA1 hash consisting of all zeros, representing the absence of a commit in Git.
 pub const ZERO_SHA: &[u8] = b"0000000000000000000000000000000000000000";
@@ -11,7 +11,7 @@ pub const ZERO_SHA: &[u8] = b"0000000000000000000000000000000000000000";
 impl RemoteGitProber {
     /// Create a new RemoteGitProber, returning None if the Git plugin is not available.
     pub fn new() -> Option<Self> {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let m = match py.import("breezy.git") {
                 Ok(m) => m,
                 Err(e) => {
@@ -31,9 +31,11 @@ impl RemoteGitProber {
     }
 }
 
-impl FromPyObject<'_> for RemoteGitProber {
-    fn extract_bound(obj: &Bound<PyAny>) -> PyResult<Self> {
-        Ok(Self(obj.clone().unbind()))
+impl<'a, 'py> FromPyObject<'a, 'py> for RemoteGitProber {
+    type Error = PyErr;
+
+    fn extract(obj: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
+        Ok(Self(obj.to_owned().unbind()))
     }
 }
 
@@ -54,18 +56,18 @@ impl std::fmt::Debug for RemoteGitProber {
 }
 
 impl crate::controldir::PyProber for RemoteGitProber {
-    fn to_object(&self, py: Python) -> PyObject {
+    fn to_object(&self, py: Python) -> Py<PyAny> {
         self.0.clone_ref(py)
     }
 }
 
 /// Format for bare local Git repositories.
-pub struct BareLocalGitControlDirFormat(PyObject);
+pub struct BareLocalGitControlDirFormat(Py<PyAny>);
 
 impl BareLocalGitControlDirFormat {
     /// Create a new BareLocalGitControlDirFormat.
     pub fn new() -> Self {
-        Python::with_gil(|py| {
+        Python::attach(|py| {
             let m = py
                 .import("breezy.git")
                 .expect("Failed to import breezy.git");
@@ -95,7 +97,7 @@ impl<'py> IntoPyObject<'py> for BareLocalGitControlDirFormat {
 
 impl crate::controldir::AsFormat for BareLocalGitControlDirFormat {
     fn as_format(&self) -> Option<crate::controldir::ControlDirFormat> {
-        Some(Python::with_gil(|py| {
+        Some(Python::attach(|py| {
             crate::controldir::ControlDirFormat::from(self.0.clone_ref(py))
         }))
     }
@@ -139,7 +141,7 @@ mod tests {
     #[test]
     fn test_remote_git_prober_into_pyobject() {
         if let Some(prober) = RemoteGitProber::new() {
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 let _pyobj = prober.into_pyobject(py).unwrap();
             });
         }
@@ -150,7 +152,7 @@ mod tests {
         let result = std::panic::catch_unwind(|| BareLocalGitControlDirFormat::new());
 
         if let Ok(format) = result {
-            Python::with_gil(|py| {
+            Python::attach(|py| {
                 let _pyobj = format.into_pyobject(py).unwrap();
             });
         }
