@@ -3,7 +3,7 @@ use pyo3::prelude::*;
 
 /// Retrieve a GitHub authentication token.
 pub fn retrieve_github_token() -> String {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let m = py.import("breezy.plugins.github.forge").unwrap();
 
         let token = m.call_method0("retrieve_github_token").unwrap();
@@ -14,7 +14,7 @@ pub fn retrieve_github_token() -> String {
 
 /// Login to GitHub using saved credentials.
 pub fn login() -> PyResult<()> {
-    Python::with_gil(|py| {
+    Python::attach(|py| {
         let m = py.import("breezy.plugins.github.cmds").unwrap();
         let cmd = m.getattr("cmd_github_login").unwrap();
 
