@@ -129,8 +129,10 @@ impl<'de> Deserialize<'de> for RevisionId {
     }
 }
 
-impl FromPyObject<'_> for RevisionId {
-    fn extract_bound(ob: &Bound<PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for RevisionId {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let bytes = ob.extract::<Vec<u8>>()?;
         Ok(Self(bytes))
     }
